unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, Grids, StdCtrls, ExtCtrls, Buttons;

//=============================================================================
//  J_[\NX
//=============================================================================
type
    TDayOfWeek = 0..6;

    TCalendar = class
    private
        FStartOfWeek: TDayOfWeek;
        FMonthOffset: integer;
        FGrid: TStringGrid;
        procedure SetMonthOffset(const ADate: TDateTime);
    protected
    public
        constructor Create(AGrid: TStringGrid; const ADate: TDateTime);
        destructor Destroy(); override;
        procedure SetDayText(const ADate: TDateTime);
        procedure SetStartOfWeek(const ADate: TDateTime; const ADayOfWeek: TDayOfWeek);
    end;

//=============================================================================
//  tH[
//=============================================================================
type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet3: TTabSheet;
    GridCalendar: TStringGrid;
    GridYear: TStringGrid;
    GridMonth: TStringGrid;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    TabSheet2: TTabSheet;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Label4: TLabel;
    Bevel1: TBevel;
    ChkShowHint: TCheckBox;
    ChkShowAAA: TCheckBox;
    BitBtn1: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure GridYearDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure GridMonthDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure GridCalendarDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure GridCalendarMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GridYearClick(Sender: TObject);
    procedure GridCalendarClick(Sender: TObject);
    procedure Label1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure ChkShowHintClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  private
    FCalendar: TCalendar;
    FDate: TDateTime;
    procedure SetGrid();
    function GetYearGridColRow(const AYear: Integer; var ACol, ARow: Integer): Boolean;
    function GetYear(const ACol, ARow: Integer): Integer;
    procedure ChangeYearMonth();
    procedure BuildDateText();
  private
    FBackShortDayNames: array[1..7] of String;
    procedure InitCondition();
  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;

implementation

uses
    DateUtils, ClipBrd, Info;

{$R *.dfm}

const
    START_YEAR: Integer = 1980;
    FINAL_YEAR: Integer = 2039;
const
    MONTHES: array[0..11] of String
    = (' 1 JAN', ' 2 FEB', ' 3 MAR', ' 4 APR',
       ' 5 MAY', ' 6 JUN', ' 7 JUL', ' 8 AUG',
       ' 9 SEP', '10 OCT', '11 NOV', '12 DEC');

    SHORTAAA: array[1..7] of String
    = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');

//=============================================================================
//  J_[\NX
//=============================================================================

//-----------------------------------------------------------------------------
//  J_[̕\̍ŏ̓Ăǂɕ\H
//  ex) ΐ؋y
//       X X 1 2 3 4 5  ŁAu1vǂɒu
//  Delphi6\Source\Samples\Calendar.pas
procedure TCalendar.SetMonthOffset(const ADate: TDateTime);
var
    FirstDate: TDateTime;
begin
    FirstDate := StartOfTheMonth(ADate);
    FMonthOffset := 2 - ((DayOfWeek(FirstDate) - FStartOfWeek + 7) mod 7);
    if FMonthOffset = 2 then FMonthOffset := -5;
end;

//-----------------------------------------------------------------------------
constructor TCalendar.Create(AGrid: TStringGrid; const ADate: TDateTime);
begin
    FGrid := AGrid;
    FStartOfWeek := 0;
    SetDayText(ADate);
end;

//-----------------------------------------------------------------------------
destructor TCalendar.Destroy();
begin
    ;
end;

//-----------------------------------------------------------------------------
procedure TCalendar.SetDayText(const ADate: TDateTime);
const
    DAY_NAME: array[1..7] of string = ('SUN', 'MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT');
    DAY_COLOR: array[1..7] of TColor = (clRed, 0, 0, 0, 0, 0, clBlue);
var
    Col, Row: integer;
    DayPos: integer;
    DayNum, MaxDay: integer;
    TempDay: TDateTime;
begin
    SetMonthOffset(ADate);

    //  j(^Cg)
    Row := 0;
    for Col := 0 to 6 do
    begin
        DayPos := (FStartOfWeek + Col) mod 7 + 1;
        FGrid.Cells[Col, Row] := DAY_NAME[DayPos];
        FGrid.Cells[Col, 7] := IntToStr(DAY_COLOR[DayPos]);
    end;
    //  t
    for Row := 1 to 6 do
    begin
        for Col := 0 to 6 do
        begin
            DayNum := FMonthOffset + Col + (Row - 1) * 7;
            MaxDay := DaysInMonth(ADate);
            //  OɓĂ
            if (DayNum < 1) then
            begin
                TempDay := StartOfTheMonth(ADate);
                TempDay := TempDay + (DayNum - 1);
                FGrid.Cells[Col, Row] := IntToStr(-Trunc(TempDay));
            end
            //  ɓĂ
            else if (DayNum > MaxDay) then
            begin
                TempDay := EndOfTheMonth(ADate);
                TempDay := TempDay + (DayNum - MaxDay);
                FGrid.Cells[Col, Row] := IntToStr(-Trunc(TempDay));
            end
            //  
            else
            begin
                TempDay := EncodeDate(YearOf(ADate), MonthOf(ADate), DayNum);
                FGrid.Cells[Col, Row] := IntToStr(Trunc(TempDay));
            end;
        end;
    end;
end;

//-----------------------------------------------------------------------------
procedure TCalendar.SetStartOfWeek(const ADate: TDateTime; const ADayOfWeek: TDayOfWeek);
begin
    FStartOfWeek := ADayOfWeek;
    SetDayText(ADate);
end;

//=============================================================================
//  tH[
//=============================================================================

//-----------------------------------------------------------------------------
procedure TMainForm.FormCreate(Sender: TObject);
var
    i: Integer;
    LGrid: TStringGrid;
begin
    //  kj\obNAbv
    for i := Low(ShortDayNames) to High(ShortDayNames) do
    begin
        FBackShortDayNames[i] := ShortDayNames[i];
    end;

    //LoadPropertyFromFile(ChangeFileExt(ParamStr(0), '.ini'), Self);
    PageControl1.ActivePageIndex := 1;
    InitCondition();

    GridYear.Canvas.Font.Assign(Self.Font);
    GridMonth.Canvas.Font.Assign(Self.Font);
    GridCalendar.Canvas.Font.Assign(Self.Font);

    FDate := Now();
    FCalendar := TCalendar.Create(GridCalendar, FDate);
    SetGrid();
    LGrid := GridCalendar;
    FDate := Abs(StrToIntDef(LGrid.Cells[LGrid.Col, LGrid.Row], 0));
    BuildDateText();
end;

//-----------------------------------------------------------------------------
//  ݒ蔽f
procedure TMainForm.InitCondition();
var
    i: Integer;
begin
    Self.ShowHint := ChkShowHint.Checked;

    if (ChkShowAAA.Checked) then
    begin
        for i := Low(ShortDayNames) to High(ShortDayNames) do
        begin
            ShortDayNames[i] := SHORTAAA[i];
        end;
    end
    else
    begin
        for i := Low(ShortDayNames) to High(ShortDayNames) do
        begin
            ShortDayNames[i] := FBackShortDayNames[i];
        end;
    end;

    BuildDateText();
end;

//-----------------------------------------------------------------------------
//  tH[j
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    //SavePropertyToFile(ChangeFileExt(ParamStr(0), '.ini'), Self);
end;

//-----------------------------------------------------------------------------
//  Nƌ̃Obh݂̕\tɑΉ悤ɑI
procedure TMainForm.SetGrid();
var
    LY, LM, LD: Word;
    LCol, LRow: Integer;
begin
    DecodeDate(FDate, LY, LM, LD);
    if (GetYearGridColRow(LY, LCol, LRow)) then
    begin
        GridYear.Col := LCol;
        GridYear.Row := LRow;
    end;
    GridMonth.Row := LM - 1;
end;

//-----------------------------------------------------------------------------
//  w肵NɑΉZ̈ʒu߂
function TMainForm.GetYearGridColRow(const AYear: Integer; var ACol, ARow: Integer): Boolean;
var
    LY: Integer;
begin
    //  512s(60NB1980`2039)
    Result := False;

    if (START_YEAR <= AYear) and (AYear <= FINAL_YEAR) then
    begin
        LY := AYear - START_YEAR;
        ACol := LY mod 5;
        ARow := LY div 5;
        Result := True;
    end;
end;

//-----------------------------------------------------------------------------
//  wZ̈ӖN߂
function TMainForm.GetYear(const ACol, ARow: Integer): Integer;
begin
    Result := START_YEAR + ACol + ARow * 5;
end;

//-----------------------------------------------------------------------------
//  NObh`
procedure TMainForm.GridYearDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
    LGrid: TStringGrid;
    LCanvas: TCanvas;
    LRect: TRect;
    LCell: String;
    LInt: Integer;
    LParams: TDrawTextParams;
    LFormat: Cardinal;
begin
    LGrid := TStringGrid(Sender);
    LCanvas := LGrid.Canvas;
    LRect := Rect;

    //  ΉN
    LInt := GetYear(ACol, ARow);
    LCell := IntToStr(LInt);

    //  ݒ
    FillChar(LParams, SizeOf(LParams), 0);
    LParams.cbSize := SizeOf(LParams);
    LFormat := DT_SINGLELINE or DT_CENTER or DT_VCENTER;

    //  wiF
    if (gdSelected in State) then
    begin
        LCanvas.Brush.Color := clSkyBlue;
    end
    else
    begin
        LCanvas.Brush.Color := clWindow;
    end;

    //  NH
    if (YearOf(Now()) = LInt) then
    begin
        LCanvas.Font.Style := LCanvas.Font.Style + [fsBold];
        LCanvas.Font.Color := clRed;
    end
    else
    begin
        LCanvas.Font.Style := LCanvas.Font.Style - [fsBold];
        LCanvas.Font.Color := clWindowText;
    end;

    //  `
    LCanvas.FillRect(LRect);
    DrawTextEx(LCanvas.Handle, PChar(LCell), -1, LRect, LFormat, @LParams);
end;

//-----------------------------------------------------------------------------
//  Obh`
procedure TMainForm.GridMonthDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
    LGrid: TStringGrid;
    LCanvas: TCanvas;
    LRect: TRect;
    LCell: String;
    LParams: TDrawTextParams;
    LFormat: Cardinal;
begin
    LGrid := TStringGrid(Sender);
    LCanvas := LGrid.Canvas;
    LRect := Rect;

    //  Ή錎
    LCell := MONTHES[ARow];

    //  ݒ
    FillChar(LParams, SizeOf(LParams), 0);
    LParams.cbSize := SizeOf(LParams);
    LFormat := DT_SINGLELINE or DT_CENTER or DT_VCENTER;

    //  wiF
    if (gdSelected in State) then
    begin
        LCanvas.Brush.Color := clSkyBlue;
    end
    else
    begin
        LCanvas.Brush.Color := clWindow;
    end;

    //  H
    if (MonthOf(Now()) = ARow + 1) then
    begin
        LCanvas.Font.Style := LCanvas.Font.Style + [fsBold];
        LCanvas.Font.Color := clRed;
    end
    else
    begin
        LCanvas.Font.Style := LCanvas.Font.Style - [fsBold];
        LCanvas.Font.Color := clWindowText;
    end;

    //  `
    LCanvas.FillRect(LRect);
    DrawTextEx(LCanvas.Handle, PChar(LCell), -1, LRect, LFormat, @LParams);
end;

//-----------------------------------------------------------------------------
//  Obh`
procedure TMainForm.GridCalendarDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
    LGrid: TStringGrid;
    LCanvas: TCanvas;
    LInt: Integer;
    LY, LM, LD: Word;
    LRect: TRect;
    LCell: String;
    LParams: TDrawTextParams;
    LFormat: Cardinal;
    LFontColor: TColor;
begin
    LGrid := TStringGrid(Sender);
    LCanvas := LGrid.Canvas;
    LRect := Rect;
    LCell := LGrid.Cells[ACol, ARow];

    LFontColor := LCanvas.Font.Color;
    LCanvas.Font.Color := clWindowText;

    //  Ή
    if not (gdFixed in State) then
    begin
        LInt := StrToIntDef(LCell, 0);
        DecodeDate(Abs(LInt), LY, LM, LD);
        LCell := Format('%2d', [LD]);

        if (LInt < 0) then
        begin
            LCanvas.Font.Color := clGrayText;
        end

        //  H
        else if (DateOf(Now()) = LInt) then
        begin
            LCanvas.Font.Style := LCanvas.Font.Style + [fsBold];
            LCanvas.Font.Color := clRed;
        end
        else
        begin
            LCanvas.Font.Style := LCanvas.Font.Style - [fsBold];
            LCanvas.Font.Color := clWindowText;
        end;
    end;

    //  ݒ
    FillChar(LParams, SizeOf(LParams), 0);
    LParams.cbSize := SizeOf(LParams);
    LFormat := DT_SINGLELINE or DT_CENTER or DT_VCENTER;

    //  wiF
    if (gdSelected in State) then
    begin
        LCanvas.Brush.Color := clSkyBlue;
    end
    else if (gdFixed in State) then
    begin
        LCanvas.Brush.Color := clGray;
    end
    else
    begin
        LCanvas.Brush.Color := clWindow;
    end;

    //  `
    LCanvas.FillRect(LRect);
    DrawTextEx(LCanvas.Handle, PChar(LCell), -1, LRect, LFormat, @LParams);

    LCanvas.Font.Color := LFontColor;
end;

//-----------------------------------------------------------------------------
//  ObhNbNŃy[W؂ւ
procedure TMainForm.GridCalendarMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if (Button = mbRight) then
    begin
        if (Sender = GridYear) then
        begin
            PageControl1.ActivePageIndex := 1;
        end
        else if (Sender = GridMonth) then
        begin
            PageControl1.ActivePageIndex := 1;
        end
        else if (Sender = GridCalendar) then
        begin
            PageControl1.ActivePageIndex := 0;
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  NύX
procedure TMainForm.ChangeYearMonth();
var
    LGrid: TStringGrid;
    LOldY, LOldM, LOldD: Word;
    LNewY, LNewM: Word;
begin
    //  ÂN
    DecodeDate(FDate, LOldY, LOldM, LOldD);

    //  VN
    LGrid := GridYear;
    LNewY := GetYear(LGrid.Col, LGrid.Row);

    //  V
    LGrid := GridMonth;
    LNewM := LGrid.Row + 1;

    //  tύX
    LGrid := GridCalendar;
    if (LOldY <> LNewY) or (LOldM <> LNewM) then
    begin
        FDate := EncodeDate(LNewY, LNewM, 1);
        FCalendar.SetDayText(FDate);
        FDate := Abs(StrToIntDef(LGrid.Cells[LGrid.Col, LGrid.Row], 0));
        BuildDateText();
    end;
end;

//-----------------------------------------------------------------------------
//  ObhNbNŔNύX
procedure TMainForm.GridYearClick(Sender: TObject);
begin
    ChangeYearMonth();
end;

//-----------------------------------------------------------------------------
//  t쐬
procedure TMainForm.BuildDateText();
begin
    Label1.Caption := FormatDateTime(Edit1.Text, FDate);
    Label2.Caption := FormatDateTime(Edit2.Text, FDate);
    Label3.Caption := FormatDateTime(Edit3.Text, FDate);
    Label4.Caption := FormatDateTime(Edit4.Text, FDate);
end;

//-----------------------------------------------------------------------------
//  ObhNbNœt쐬
procedure TMainForm.GridCalendarClick(Sender: TObject);
var
    LGrid: TStringGrid;
begin
    LGrid := TStringGrid(Sender);
    FDate := Abs(StrToIntDef(LGrid.Cells[LGrid.Col, LGrid.Row], 0));
    BuildDateText();
end;

//-----------------------------------------------------------------------------
//  tH[}bgύXœt쐬
procedure TMainForm.Edit1Change(Sender: TObject);
begin
    BuildDateText();
end;

//-----------------------------------------------------------------------------
//  xNbNŃLvVNbv{[hɃRs[
procedure TMainForm.Label1Click(Sender: TObject);
begin
    Clipboard.AsText := TLabel(Sender).Caption;
end;

//-----------------------------------------------------------------------------
//  qg\
procedure TMainForm.ChkShowHintClick(Sender: TObject);
begin
    InitCondition();
end;

procedure TMainForm.BitBtn1Click(Sender: TObject);
begin
    OpenInfoForm();
end;

end.
 